@@ -23,7 +23,11 @@ from mcp.types import (
 
 from mcp_agent.core.context import Context
 from mcp_agent.tracing.semconv import GEN_AI_AGENT_NAME, GEN_AI_TOOL_NAME
-from mcp_agent.tracing.telemetry import get_tracer, record_attributes
+from mcp_agent.tracing.telemetry import (
+    annotate_span_for_call_tool_result,
+    get_tracer,
+    record_attributes,
+)
 from mcp_agent.mcp.mcp_agent_client_session import MCPAgentClientSession
 from mcp_agent.mcp.mcp_aggregator import (
     MCPAggregator,
@@ -144,25 +148,11 @@ class Agent(BaseModel):
     # endregion
 
     def model_post_init(self, __context) -> None:
-        if self.context is None:
-            # Fall back to global context if available
-            from mcp_agent.core.context import get_current_context
-
-            self.context = get_current_context()
-
         # Map function names to tools
         self._function_tool_map = {
             (tool := FastTool.from_function(fn)).name: tool for fn in self.functions
         }
 
-        # Default human_input_callback from context, if absent
-        if self.human_input_callback is None:
-            ctx_handler = getattr(self.context, "human_input_handler", None)
-            if ctx_handler is not None:
-                self.human_input_callback = ctx_handler
-
-        self._agent_tasks = AgentTasks(self.context)
-
     async def attach_llm(
         self, llm_factory: Callable[..., LLM] | None = None, llm: LLM | None = None
     ) -> LLM:
@@ -203,6 +193,16 @@ class Agent(BaseModel):
 
     async def initialize(self, force: bool = False):
         """Initialize the agent."""
+
+        if self.initialized and not force:
+            return
+
+        if self.context is None:
+            # Fall back to global context if available
+            from mcp_agent.core.context import get_current_context
+
+            self.context = get_current_context()
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.initialize"
@@ -213,12 +213,17 @@ class Agent(BaseModel):
             span.set_attribute("force", force)
 
             async with self._init_lock:
-                if self.initialized and not force:
-                    return
-
                 span.add_event("initialize_start")
                 logger.debug(f"Initializing agent {self.name}...")
 
+                if self._agent_tasks is None:
+                    self._agent_tasks = AgentTasks(self.context)
+
+                if self.human_input_callback is None:
+                    ctx_handler = getattr(self.context, "human_input_handler", None)
+                    if ctx_handler is not None:
+                        self.human_input_callback = ctx_handler
+
                 executor = self.context.executor
 
                 result: InitAggregatorResponse = await executor.execute(
@@ -267,6 +272,10 @@ class Agent(BaseModel):
         """
         logger.debug(f"Shutting down agent {self.name}...")
 
+        if not self.initialized:
+            logger.debug(f"Agent {self.name} is not initialized, skipping shutdown.")
+            return
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.shutdown"
@@ -310,6 +319,9 @@ class Agent(BaseModel):
         """
         Get the capabilities of a specific server.
         """
+        if not self.initialized:
+            await self.initialize()
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.get_capabilities"
@@ -317,9 +329,6 @@ class Agent(BaseModel):
             span.set_attribute(GEN_AI_AGENT_NAME, self.name)
             span.set_attribute("initialized", self.initialized)
 
-            if not self.initialized:
-                await self.initialize()
-
             executor = self.context.executor
             result: Dict[str, ServerCapabilities] = await executor.execute(
                 self._agent_tasks.get_capabilities_task,
@@ -364,6 +373,8 @@ class Agent(BaseModel):
         """
         Get the session data of a specific server.
         """
+        if not self.initialized:
+            await self.initialize()
 
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
@@ -372,9 +383,6 @@ class Agent(BaseModel):
             span.set_attribute(GEN_AI_AGENT_NAME, self.name)
             span.set_attribute("initialized", self.initialized)
 
-            if not self.initialized:
-                await self.initialize()
-
             executor = self.context.executor
             result: GetServerSessionResponse = await executor.execute(
                 self._agent_tasks.get_server_session,
@@ -384,6 +392,9 @@ class Agent(BaseModel):
             return result
 
     async def list_tools(self, server_name: str | None = None) -> ListToolsResult:
+        if not self.initialized:
+            await self.initialize()
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.list_tools"
@@ -394,9 +405,6 @@ class Agent(BaseModel):
                 "human_input_callback", self.human_input_callback is not None
             )
 
-            if not self.initialized:
-                await self.initialize()
-
             if server_name:
                 span.set_attribute("server_name", server_name)
                 result = ListToolsResult(
@@ -482,6 +490,9 @@ class Agent(BaseModel):
         """
         List resources available to the agent from MCP servers.
         """
+        if not self.initialized:
+            await self.initialize()
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.list_resources"
@@ -491,9 +502,6 @@ class Agent(BaseModel):
             if server_name:
                 span.set_attribute("server_name", server_name)
 
-            if not self.initialized:
-                await self.initialize()
-
             executor = self.context.executor
             result: ListResourcesResult = await executor.execute(
                 self._agent_tasks.list_resources_task,
@@ -505,6 +513,9 @@ class Agent(BaseModel):
         """
         Read a resource from an MCP server.
         """
+        if not self.initialized:
+            await self.initialize()
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.read_resource"
@@ -515,9 +526,6 @@ class Agent(BaseModel):
             if server_name:
                 span.set_attribute("server_name", server_name)
 
-            if not self.initialized:
-                await self.initialize()
-
             executor = self.context.executor
             result: ReadResourceResult = await executor.execute(
                 self._agent_tasks.read_resource_task,
@@ -621,6 +629,10 @@ class Agent(BaseModel):
         return messages
 
     async def list_prompts(self, server_name: str | None = None) -> ListPromptsResult:
+        # Check if the agent is initialized
+        if not self.initialized:
+            await self.initialize()
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.list_prompts"
@@ -631,14 +643,10 @@ class Agent(BaseModel):
             if server_name:
                 span.set_attribute("server_name", server_name)
 
-            # Check if the agent is initialized
-            if not self.initialized:
-                await self.initialize()
-
             executor = self.context.executor
             result: ListPromptsResult = await executor.execute(
                 self._agent_tasks.list_prompts_task,
-                ListToolsRequest(agent_name=self.name, server_name=server_name),
+                ListPromptsRequest(agent_name=self.name, server_name=server_name),
             )
 
             if self.context.tracing_enabled:
@@ -670,6 +678,9 @@ class Agent(BaseModel):
         arguments: dict[str, str] | None = None,
         server_name: str | None = None,
     ) -> GetPromptResult:
+        if not self.initialized:
+            await self.initialize()
+
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
             f"{self.__class__.__name__}.{self.name}.get_prompt"
@@ -680,9 +691,6 @@ class Agent(BaseModel):
                 span.set_attribute("initialized", self.initialized)
                 record_attributes(span, arguments, "arguments")
 
-            if not self.initialized:
-                await self.initialize()
-
             executor = self.context.executor
             result: GetPromptResult = await executor.execute(
                 self._agent_tasks.get_prompt_task,
@@ -828,6 +836,8 @@ class Agent(BaseModel):
         self, name: str, arguments: dict | None = None, server_name: str | None = None
     ) -> CallToolResult:
         # Call the tool on the server
+        if not self.initialized:
+            await self.initialize()
 
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
@@ -844,28 +854,10 @@ class Agent(BaseModel):
                 if arguments is not None:
                     record_attributes(span, arguments, "arguments")
 
-            if not self.initialized:
-                await self.initialize()
-
             def _annotate_span_for_result(result: CallToolResult):
                 if not self.context.tracing_enabled:
                     return
-                span.set_attribute("result.isError", result.isError)
-                if result.isError:
-                    span.set_status(trace.Status(trace.StatusCode.ERROR))
-                    error_message = (
-                        result.content[0].text
-                        if len(result.content) > 0 and result.content[0].type == "text"
-                        else "Error calling tool"
-                    )
-                    span.record_exception(Exception(error_message))
-                else:
-                    for idx, content in enumerate(result.content):
-                        span.set_attribute(f"result.content.{idx}.type", content.type)
-                        if content.type == "text":
-                            span.set_attribute(
-                                f"result.content.{idx}.text", result.content[idx].text
-                            )
+                annotate_span_for_call_tool_result(span, result)
 
             if name == HUMAN_INPUT_TOOL_NAME:
                 # Call the human input tool
