@@ -131,6 +131,12 @@ class FileTransport(FilteredEventTransport):
         self.encoding = encoding
         self._serializer = JSONSerializer()
 
+        # Batching for efficient writes
+        self._write_buffer: List[str] = []
+        self._buffer_lock = asyncio.Lock()
+        self._flush_task: asyncio.Task | None = None
+        self._running = True
+
         # Create directory if it doesn't exist
         self.filepath.parent.mkdir(parents=True, exist_ok=True)
 
@@ -156,15 +162,27 @@ class FileTransport(FilteredEventTransport):
         if event.data:
             log_entry["data"] = self._serializer(event.data)
 
+        # Prepare the log line
+        log_line = json.dumps(log_entry, separators=(",", ":")) + "\n"
+
+        # Use asyncio to run file I/O in executor to avoid blocking
         try:
-            with open(self.filepath, mode=self.mode, encoding=self.encoding) as f:
-                # Write the log entry as compact JSON (JSONL format)
-                f.write(json.dumps(log_entry, separators=(",", ":")) + "\n")
-                f.flush()  # Ensure writing to disk
+            loop = asyncio.get_event_loop()
+            await loop.run_in_executor(
+                None,  # Use default executor
+                self._write_to_file,
+                log_line,
+            )
         except IOError as e:
             # Log error without recursion
             print(f"Error writing to log file {self.filepath}: {e}")
 
+    def _write_to_file(self, log_line: str) -> None:
+        """Synchronous file write helper for use in executor."""
+        with open(self.filepath, mode=self.mode, encoding=self.encoding) as f:
+            f.write(log_line)
+            f.flush()  # Ensure writing to disk
+
     async def close(self) -> None:
         """Clean up resources if needed."""
         pass  # File handles are automatically closed after each write
@@ -306,25 +324,30 @@ class AsyncEventBus:
         if cls._instance:
             # Signal shutdown
             cls._instance._running = False
-            cls._instance._stop_event.set()
+            if hasattr(cls._instance, "_stop_event"):
+                cls._instance._stop_event.set()
 
             # Clear the singleton instance
             cls._instance = None
 
     async def start(self):
         """Start the event bus and all lifecycle-aware listeners."""
-        if self._running:
-            return
-        self.init_queue()
-        # Start each lifecycle-aware listener
+        # Always ensure queue is initialized
+        if not hasattr(self, "_queue"):
+            self.init_queue()
+
+        # Start each lifecycle-aware listener (even if already running)
+        # This ensures listeners are started even if auto-start happened
         for listener in self.listeners.values():
             if isinstance(listener, LifecycleAwareListener):
                 await listener.start()
 
-        # Clear stop event and start processing
-        self._stop_event.clear()
-        self._running = True
-        self._task = asyncio.create_task(self._process_events())
+        # If not already running, start the event processing task
+        if not self._running:
+            # Clear stop event and start processing
+            self._stop_event.clear()
+            self._running = True
+            self._task = asyncio.create_task(self._process_events())
 
     async def stop(self):
         """Stop the event bus and all lifecycle-aware listeners."""
@@ -333,10 +356,11 @@ class AsyncEventBus:
 
         # Signal processing to stop
         self._running = False
-        self._stop_event.set()
+        if hasattr(self, "_stop_event"):
+            self._stop_event.set()
 
-        # Try to process remaining items with a timeout
-        if not self._queue.empty():
+        # Try to process remaining items with a timeout if queue exists
+        if hasattr(self, "_queue") and not self._queue.empty():
             try:
                 # Give some time for remaining items to be processed
                 await asyncio.wait_for(self._queue.join(), timeout=5.0)
@@ -389,9 +413,24 @@ class AsyncEventBus:
         except Exception as e:
             print(f"Error in transport.send_event: {e}")
 
+        # Initialize queue and start processing if needed
+        if not hasattr(self, "_queue"):
+            self.init_queue()
+            # Auto-start the event processing task if not running
+            if not self._running:
+                self._running = True
+                self._task = asyncio.create_task(self._process_events())
+
         # Then queue for listeners
         await self._queue.put(event)
 
+    async def _send_to_transport(self, event: Event):
+        """Send event to transport with error handling."""
+        try:
+            await self.transport.send_event(event)
+        except Exception as e:
+            print(f"Error in transport.send_event: {e}")
+
     def add_listener(self, name: str, listener: EventListener):
         """Add a listener to the event bus."""
         self.listeners[name] = listener
@@ -405,18 +444,38 @@ class AsyncEventBus:
         while self._running:
             event = None
             try:
-                # Use wait_for with a timeout to allow checking running state
+                # Use wait with both queue.get() and stop_event.wait() to avoid timeout delays
                 try:
                     # Check if we should be stopping first
                     if not self._running or self._stop_event.is_set():
                         break
 
-                    event = await asyncio.wait_for(self._queue.get(), timeout=0.1)
-                except asyncio.TimeoutError:
-                    # Check again before continuing
-                    if not self._running or self._stop_event.is_set():
+                    # Wait for either an event or stop signal without timeout
+                    queue_task = asyncio.create_task(self._queue.get())
+                    stop_task = asyncio.create_task(self._stop_event.wait())
+
+                    done, pending = await asyncio.wait(
+                        [queue_task, stop_task], return_when=asyncio.FIRST_COMPLETED
+                    )
+
+                    # Cancel pending tasks
+                    for task in pending:
+                        task.cancel()
+                        try:
+                            await task
+                        except asyncio.CancelledError:
+                            pass
+
+                    # Check which task completed
+                    if stop_task in done:
                         break
-                    continue
+
+                    if queue_task in done:
+                        event = queue_task.result()
+                    else:
+                        continue
+                except asyncio.CancelledError:
+                    break
 
                 # Process the event through all listeners
                 tasks = []
@@ -445,21 +504,22 @@ class AsyncEventBus:
                 if event is not None:
                     self._queue.task_done()
 
-        # Process remaining events in queue
-        while not self._queue.empty():
-            try:
-                event = self._queue.get_nowait()
-                tasks = []
-                for listener in self.listeners.values():
-                    try:
-                        tasks.append(listener.handle_event(event))
-                    except Exception:
-                        pass
-                if tasks:
-                    await asyncio.gather(*tasks, return_exceptions=True)
-                self._queue.task_done()
-            except asyncio.QueueEmpty:
-                break
+        # Process remaining events in queue if it exists
+        if hasattr(self, "_queue"):
+            while not self._queue.empty():
+                try:
+                    event = self._queue.get_nowait()
+                    tasks = []
+                    for listener in self.listeners.values():
+                        try:
+                            tasks.append(listener.handle_event(event))
+                        except Exception:
+                            pass
+                    if tasks:
+                        await asyncio.gather(*tasks, return_exceptions=True)
+                    self._queue.task_done()
+                except asyncio.QueueEmpty:
+                    break
 
 
 class MultiTransport(EventTransport):
