@@ -1,3 +1,5 @@
+import asyncio
+import functools
 import json
 from typing import Any, Iterable, Optional, Type, Union
 from azure.ai.inference import ChatCompletionsClient
@@ -47,6 +49,7 @@ from mcp_agent.tracing.semconv import (
     GEN_AI_USAGE_OUTPUT_TOKENS,
 )
 from mcp_agent.tracing.telemetry import get_tracer
+from mcp_agent.tracing.token_tracking_decorator import track_tokens
 from mcp_agent.utils.common import typed_dict_extras
 from mcp_agent.workflows.llm.augmented_llm import (
     AugmentedLLM,
@@ -89,7 +92,7 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, type_converter=MCPAzureTypeConverter, **kwargs)
 
-        self.provider = "Microsoft Azure"
+        self.provider = "Azure"
         # Initialize logger with name if available
         self.logger = get_logger(f"{__name__}.{self.name}" if self.name else __name__)
 
@@ -124,10 +127,11 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
             use_history=True,
         )
 
+    @track_tokens()
     async def generate(self, message, request_params: RequestParams | None = None):
         """
         Process a query using an LLM and available tools.
-        The default implementation uses Azure OpenAI 4o-mini as the LLM.
+        The default implementation uses Azure OpenAI 5 as the LLM.
         Override this method to use a different LLM.
         """
         tracer = get_tracer(self.context)
@@ -280,6 +284,15 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
                     )
                     span.set_attributes(response_data)
 
+            # Record token usage in context token counter
+            if self.context.token_counter:
+                await self.context.token_counter.record_usage(
+                    input_tokens=total_input_tokens,
+                    output_tokens=total_output_tokens,
+                    model_name=model,
+                    provider=self.provider,
+                )
+
             return responses
 
     async def generate_str(
@@ -518,7 +531,11 @@ class AzureCompletionTasks:
             )
 
         payload = request.payload
-        response = azure_client.complete(**payload)
+        # Offload sync SDK call to a thread to avoid blocking the event loop
+        loop = asyncio.get_running_loop()
+        response = await loop.run_in_executor(
+            None, functools.partial(azure_client.complete, **payload)
+        )
         return response
 
 
