@@ -30,6 +30,7 @@ from mcp_agent.workflows.llm.augmented_llm import (
     CallToolResult,
 )
 from mcp_agent.workflows.llm.multipart_converter_google import GoogleConverter
+from mcp_agent.tracing.token_tracking_decorator import track_tokens
 
 
 class GoogleAugmentedLLM(
@@ -46,7 +47,7 @@ class GoogleAugmentedLLM(
     def __init__(self, *args, **kwargs):
         super().__init__(*args, type_converter=GoogleMCPTypeConverter, **kwargs)
 
-        self.provider = "Google (AI Studio)"
+        self.provider = "Google (AI_Studio)"
         # Initialize logger with name if available
         self.logger = get_logger(f"{__name__}.{self.name}" if self.name else __name__)
 
@@ -72,6 +73,7 @@ class GoogleAugmentedLLM(
             use_history=True,
         )
 
+    @track_tokens()
     async def generate(self, message, request_params: RequestParams | None = None):
         """
         Process a query using an LLM and available tools.
