@@ -6,7 +6,7 @@ from typing import Any, Dict, Iterable, List, Type, cast
 from pydantic import BaseModel
 
 
-from openai import OpenAI, AsyncOpenAI
+from openai import AsyncOpenAI
 from openai.types.chat import (
     ChatCompletionAssistantMessageParam,
     ChatCompletionContentPartParam,
@@ -38,6 +38,7 @@ from mcp.types import (
 from mcp_agent.config import OpenAISettings
 from mcp_agent.executor.workflow_task import workflow_task
 from mcp_agent.tracing.telemetry import get_tracer, telemetry
+from mcp_agent.tracing.token_tracking_decorator import track_tokens
 from mcp_agent.tracing.semconv import (
     GEN_AI_AGENT_NAME,
     GEN_AI_REQUEST_MODEL,
@@ -113,7 +114,9 @@ class OpenAIAugmentedLLM(
             if hasattr(self.context.config.openai, "reasoning_effort"):
                 self._reasoning_effort = self.context.config.openai.reasoning_effort
 
-        self._reasoning = lambda model: model.startswith(("o1", "o3", "o4"))
+        self._reasoning = lambda model: model and model.startswith(
+            ("o1", "o3", "o4", "gpt-5")
+        )
 
         if self._reasoning(default_model):
             self.logger.info(
@@ -148,6 +151,7 @@ class OpenAIAugmentedLLM(
 
         return ChatCompletionAssistantMessageParam(**assistant_message_params)
 
+    @track_tokens()
     async def generate(
         self,
         message,
@@ -369,6 +373,15 @@ class OpenAIAugmentedLLM(
                     )
                     span.set_attributes(response_data)
 
+            # Record token usage in context token counter
+            if self.context.token_counter:
+                await self.context.token_counter.record_usage(
+                    input_tokens=total_input_tokens,
+                    output_tokens=total_output_tokens,
+                    model_name=model,
+                    provider=self.provider,
+                )
+
             return responses
 
     async def generate_str(
@@ -491,10 +504,10 @@ class OpenAIAugmentedLLM(
     async def execute_tool_call(
         self,
         tool_call: ChatCompletionMessageToolCall,
-    ) -> ChatCompletionToolMessageParam | None:
+    ) -> ChatCompletionToolMessageParam:
         """
         Execute a single tool call and return the result message.
-        Returns None if there's no content to add to messages.
+        Returns a single ChatCompletionToolMessageParam object.
         """
         tracer = get_tracer(self.context)
         with tracer.start_as_current_span(
@@ -533,16 +546,11 @@ class OpenAIAugmentedLLM(
 
             self._annotate_span_for_call_tool_result(span, result)
 
-            if result.content:
-                return ChatCompletionToolMessageParam(
-                    role="tool",
-                    tool_call_id=tool_call_id,
-                    content=[
-                        mcp_content_to_openai_content_part(c) for c in result.content
-                    ],
-                )
-
-            return None
+            return ChatCompletionToolMessageParam(
+                role="tool",
+                tool_call_id=tool_call_id,
+                content=[mcp_content_to_openai_content_part(c) for c in result.content],
+            )
 
     def message_param_str(self, message: ChatCompletionMessageParam) -> str:
         """Convert an input message to a string representation."""
@@ -856,8 +864,7 @@ class OpenAICompletionTasks:
         """
         Request a completion from OpenAI's API.
         """
-
-        openai_client = OpenAI(
+        async_openai_client = AsyncOpenAI(
             api_key=request.config.api_key,
             base_url=request.config.base_url,
             http_client=request.config.http_client
@@ -869,7 +876,7 @@ class OpenAICompletionTasks:
         )
 
         payload = request.payload
-        response = openai_client.chat.completions.create(**payload)
+        response = await async_openai_client.chat.completions.create(**payload)
         response = ensure_serializable(response)
         return response
 
@@ -1063,7 +1070,8 @@ def mcp_content_to_openai_content_part(
         return ChatCompletionContentPartTextParam(type="text", text=content.text)
     elif isinstance(content, ImageContent):
         return ChatCompletionContentPartImageParam(
-            type="image_url", image_url=f"data:{content.mimeType};base64,{content.data}"
+            type="image_url",
+            image_url={"url": f"data:{content.mimeType};base64,{content.data}"},
         )
     elif isinstance(content, EmbeddedResource):
         if isinstance(content.resource, TextResourceContents):
@@ -1076,7 +1084,9 @@ def mcp_content_to_openai_content_part(
             ):
                 return ChatCompletionContentPartImageParam(
                     type="image_url",
-                    image_url=f"data:{content.resource.mimeType};base64,{content.resource.blob}",
+                    image_url={
+                        "url": f"data:{content.resource.mimeType};base64,{content.resource.blob}"
+                    },
                 )
             else:
                 # Best effort if mime type is unknown
